<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:b="http://www.schema-library.org/add/refed/by/substitutionGroup"
                xmlns:c="http://www.schema-library.org/add/refed/by/substitutionGroup/local"
                xmlns:sic="http://www.schema-library.org/schema-in-context"
                xmlns:xs="http://www.w3.org/2001/XMLSchema"
                exclude-result-prefixes="#all" 
                version="2.0">
    
    <!-- 
        An element may be substitutable by other elements. This
        program adds a substitutable-by-these-elements attribute to 
        an element if it can be substituted. The value of 
        substitutable-by-these-elements is a space-separated list of 
        idref values - one idref value to each element that can substitute
        for the element.
        
        Author: Roger L. Costello
    -->
    
    <xsl:function name="b:add-refed-by-substitutionGroup-to-items-in-all-schemas" as="document-node()">
        <xsl:param name="schemas" as="document-node()" />
        
        <xsl:variable name="schema" select="$schemas/schemas/xs:schema[1]" />
        <xsl:variable name="schema-in-context" select="sic:schema-in-context($schemas/schemas/xs:schema[1])" />
        
        <xsl:variable name="updated-schemas">
            <xsl:sequence select="b:add-refed-by-substitutionGroup-to-items-in-all-schemas($schema, $schema-in-context)" />
        </xsl:variable>
        
        <xsl:sequence select="$updated-schemas" />
        
    </xsl:function>
    
    <xsl:function name="b:add-refed-by-substitutionGroup-to-items-in-all-schemas" as="document-node()">
        <xsl:param name="schema"/>
        <xsl:param name="schema-in-context"/>
        
        
        <xsl:variable name="schema-item" as="element()*">
            <container>
                <item><xsl:sequence select="$schema" /></item>
                <schema-in-context><xsl:sequence select="$schema-in-context" /></schema-in-context>
            </container>
        </xsl:variable>
        
        <xsl:variable name="updated-schemas">
            <schemas> 
                <xsl:apply-templates select="$process-graph">
                    <xsl:with-param name="items-to-process" select="$schema-item" />
                    <xsl:with-param name="items-already-processed-or-in-queue-for-processing" select="$schema-item" />
                    <xsl:with-param name="function-to-process-current-item" select="$b:function-to-process-current-schema" />
                    <xsl:with-param name="function-to-determine-new-items-to-process" select="$b:function-to-determine-new-schemas-to-process" />
                </xsl:apply-templates> 
            </schemas>  
        </xsl:variable>
        
        <xsl:sequence select="$updated-schemas" />
        
    </xsl:function>
    
    
    <xsl:variable name="b:function-to-determine-new-schemas-to-process" select="document('')//b:function-to-determine-new-schemas-to-process[1]" />
    <xsl:variable name="b:function-to-process-current-schema" select="document('')//b:function-to-process-current-schema[1]" />
    
    <b:function-to-determine-new-schemas-to-process />
    <b:function-to-process-current-schema />
    
    <xsl:template match="b:function-to-determine-new-schemas-to-process">
        <xsl:param name="item" />
        <xsl:param name="items-already-processed-or-in-queue-for-processing" />
        
        <xsl:variable name="schema-in-context" select="$item/schema-in-context/*" />
        <xsl:variable name="items" select="$item/item/xs:schema/(xs:include[@schemaLocation] | xs:import[@schemaLocation] | xs:redefine[@schemaLocation])" />
        <xsl:sequence select="b:helper-function-to-determine-new-schemas-to-process($schema-in-context, $items, $items-already-processed-or-in-queue-for-processing)" />
        
    </xsl:template>
    
    <xsl:function name="b:helper-function-to-determine-new-schemas-to-process">
        <xsl:param name="schema-in-context" />
        <xsl:param name="items" />
        <xsl:param name="items-already-processed-or-in-queue-for-processing" />
        
        <xsl:choose>
            <xsl:when test="empty($items)" />
            <xsl:otherwise>
                <xsl:for-each select="$items[1]">
                    
                    <xsl:variable name="i" select="." />
                    <xsl:variable name="referenced-schema" select="$schemas-after-adding-idref-substitutionGroupRef-and-targetNamespace/schemas/xs:schema[@_id eq $i/@idref]" />
                    
                    <xsl:choose>
                        <xsl:when test="not($referenced-schema/@_id = $items-already-processed-or-in-queue-for-processing/item/xs:schema/@_id)">
                            <xsl:variable name="schema-item" as="element()*">
                                <container>
                                    <item><xsl:sequence select="$referenced-schema" /></item>
                                    <schema-in-context><xsl:sequence select="sic:schema-in-context($referenced-schema, $schema-in-context)" /></schema-in-context>
                                </container>
                            </xsl:variable>
                            <xsl:sequence select="$schema-item" />
                            <xsl:sequence select="b:helper-function-to-determine-new-schemas-to-process($schema-in-context, $items[position() gt 1], ($items-already-processed-or-in-queue-for-processing, $schema-item))" />
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:sequence select="b:helper-function-to-determine-new-schemas-to-process($schema-in-context, $items[position() gt 1], $items-already-processed-or-in-queue-for-processing)" />
                        </xsl:otherwise>
                    </xsl:choose>
                    
                </xsl:for-each>
            </xsl:otherwise>
        </xsl:choose>
        
    </xsl:function>
    

    <xsl:template match="b:function-to-process-current-schema">
        <xsl:param name="item" />
        
        <xsl:variable name="schema" select="$item/item/*" />
        <xsl:variable name="schema-in-context" select="$item/schema-in-context/*" />
        
        <xsl:sequence select="b:add-refed-by-substitutionGroup-to-schema-items($schema, $schema-in-context)" />
        
    </xsl:template> 
    

    <xsl:function name="b:add-refed-by-substitutionGroup-to-schema-items">
        <xsl:param name="schema"/>
        <xsl:param name="schema-in-context"/>
        
        <xsl:apply-templates select="$schema" mode="add-refed-by-substitutionGroup">
            <xsl:with-param name="schema" select="$schema" />
            <xsl:with-param name="schema-in-context" select="$schema-in-context" />
        </xsl:apply-templates>
        
     </xsl:function>
    
    <xsl:template match="/" mode="add-refed-by-substitutionGroup">
        <xsl:param name="schema"/>
        <xsl:param name="schema-in-context"/>
        
        <xsl:copy>
            <xsl:apply-templates select="node()"  mode="add-refed-by-substitutionGroup">
                <xsl:with-param name="schema" select="$schema" />
                <xsl:with-param name="schema-in-context" select="$schema-in-context" />
            </xsl:apply-templates>
        </xsl:copy>
        
    </xsl:template>
    
    <xsl:template match="xs:schema" mode="add-refed-by-substitutionGroup">
        <xsl:param name="schema"/>
        <xsl:param name="schema-in-context"/>
        
        <xsl:copy>
            <xsl:copy-of select="@*" />
            <xsl:apply-templates select="node()"  mode="add-refed-by-substitutionGroup">
                <xsl:with-param name="schema" select="$schema" />
                <xsl:with-param name="schema-in-context" select="$schema-in-context" />
            </xsl:apply-templates>
        </xsl:copy>
        
    </xsl:template>
    
    <xsl:template match="node()" mode="add-refed-by-substitutionGroup">
        <xsl:param name="schema"/>
        <xsl:param name="schema-in-context"/>
        
        <xsl:copy-of select="." />
        
    </xsl:template>
    
    <xsl:template match="xs:schema/xs:element" mode="add-refed-by-substitutionGroup">
        <xsl:param name="schema"/>
        <xsl:param name="schema-in-context"/>
        
        <xsl:variable name="myID" select="@_id" />
        
        <xsl:variable name="substitutableBy" select="$schemas-after-adding-idref-substitutionGroupRef-and-targetNamespace/descendant-or-self::xs:schema/xs:element[@substitutionGroupRef][@substitutionGroupRef eq $myID]" />
        
        <xsl:choose>
            <xsl:when test="exists($substitutableBy)">
                <xsl:variable name="ids-of-elements-that-can-substitute-this-element" select="b:get-ids-of-elements-that-can-substitute-this-element(., $schema-in-context)" />
                <xsl:copy>
                    <xsl:copy-of select="@*" />
                    <xsl:attribute name="substitutable-by-these-elements"><xsl:value-of select="string-join($ids-of-elements-that-can-substitute-this-element, ' ')" /></xsl:attribute>
                    <xsl:copy-of select="* | node()" />
                </xsl:copy>
            </xsl:when>
            <xsl:otherwise>
                <xsl:copy-of select="." />
            </xsl:otherwise>
        </xsl:choose>
        
    </xsl:template>
    
    
    <xsl:function name="b:get-ids-of-elements-that-can-substitute-this-element">
        <xsl:param name="element" />
        <xsl:param name="schema-in-context" />
        
        <xsl:variable name="myID" select="$element/@_id" />
        
        <xsl:variable name="substitutableBy" select="$schemas-after-adding-idref-substitutionGroupRef-and-targetNamespace/descendant-or-self::xs:schema/xs:element[@substitutionGroupRef][@substitutionGroupRef eq $myID]" />
        
        <xsl:variable name="substitutable-by-these-elements" as="element()*">
            <xsl:for-each select="$substitutableBy">
                <container>
                    <element><xsl:sequence select="." /></element>
                    <schema-in-context><xsl:sequence select="$schema-in-context" /></schema-in-context>
                </container>
            </xsl:for-each>
        </xsl:variable>
        
        <xsl:apply-templates select="$process-graph">
            <xsl:with-param name="items-to-process" select="$substitutable-by-these-elements" />
            <xsl:with-param name="items-already-processed-or-in-queue-for-processing" select="$substitutable-by-these-elements" />
            <xsl:with-param name="function-to-process-current-item" select="$c:function-to-process-current-schema" />
            <xsl:with-param name="function-to-determine-new-items-to-process" select="$c:function-to-determine-new-schemas-to-process" />
        </xsl:apply-templates>
        
    </xsl:function>
    
    
    <xsl:variable name="c:function-to-determine-new-schemas-to-process" select="document('')//c:function-to-determine-new-schemas-to-process[1]" />
    <xsl:variable name="c:function-to-process-current-schema" select="document('')//c:function-to-process-current-schema[1]" />
    
    <c:function-to-determine-new-schemas-to-process />
    <c:function-to-process-current-schema />
    
    
    <!-- 
        The new schemas are those pointed at by xs:include, xs:import, and xs:redefine which are not already on
        the list for processing.
    -->
    
    <xsl:template match="c:function-to-determine-new-schemas-to-process">
        <xsl:param name="item" />
        <xsl:param name="items-already-processed-or-in-queue-for-processing" />
        
        <xsl:variable name="element" select="$item/element/*" />
        <xsl:variable name="schema-in-context" select="$item/schema-in-context/*" />
        
        <xsl:variable name="myID" select="$element/@_id" />
        
        <xsl:variable name="substitutableBy" select="$schemas-after-adding-idref-substitutionGroupRef-and-targetNamespace/descendant-or-self::xs:schema/xs:element[@substitutionGroupRef][@substitutionGroupRef eq $myID]" />
        
        <xsl:for-each select="$substitutableBy">
            <container>
                <element><xsl:sequence select="." /></element>
                <schema-in-context><xsl:sequence select="$schema-in-context" /></schema-in-context>
            </container>
        </xsl:for-each>
        
    </xsl:template>
    
    
    <xsl:template match="c:function-to-process-current-schema">
        <xsl:param name="item" />
        
        <xsl:variable name="element" select="$item/element/*" />
        <xsl:variable name="schema-in-context" select="$item/schema-in-context/*" />
        
        <xsl:value-of select="$element/@_id" />
        
    </xsl:template> 
     
</xsl:stylesheet>
