<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
                xmlns:m="http://www.schema-library.org/mixed-content/restrict" 
                xmlns:xs="http://www.w3.org/2001/XMLSchema"
                exclude-result-prefixes="#all" 
                version="2.0">

    <!-- 
        This program deals with a complexType with simpleContent. 
        See my paper "Compendium of complexTypes with simpleContent"
        You will see that this program does nothing (i.e., it does
        an identity transform). The reason is that I realized that
        if I convert complexTypes into simpleTypes (for example) then
        it will break a complexType that extends the orginal complexType.
        So there really isn't anything that can be done with complexTypes
        with simpleContent.
        
        Author: Roger L. Costello
    -->

    <xsl:key name="ids" match="*[@_id]" use="@_id"/>

    <xsl:function name="m:complexTypes-with-simpleContent" as="document-node()">
        <xsl:param name="schemas" as="document-node()"/>
        
        <xsl:variable name="updated-schemas">
            <schemas>
                <xsl:apply-templates select="$schemas/schemas/xs:schema" mode="complexTypes-with-simpleContent"/>
            </schemas>
        </xsl:variable>

        <xsl:sequence select="$updated-schemas"/>

    </xsl:function>

    <xsl:template match="xs:schema" mode="complexTypes-with-simpleContent">
        <xsl:copy>
            <xsl:copy-of select="@*"/>
            <xsl:apply-templates mode="complexTypes-with-simpleContent"/>
        </xsl:copy>

    </xsl:template>
    
    
    <xsl:template match="node()" mode="complexTypes-with-simpleContent">
        <xsl:copy>
            <xsl:copy-of select="@*"/>
            <xsl:apply-templates mode="complexTypes-with-simpleContent"/>
        </xsl:copy>
        
    </xsl:template>

</xsl:stylesheet>
