<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
                xmlns:m="http://www.schema-library.org/mixed-content/restrict" 
                xmlns:xs="http://www.w3.org/2001/XMLSchema"
                exclude-result-prefixes="#all" 
                version="2.0">

    <!-- 
        This program deals with a complexType with simpleContent. 
        See my paper "Compendium of complexTypes with simpleContent"
        
        Author: Roger L. Costello
    -->

    <xsl:key name="ids" match="*[@_id]" use="@_id"/>

    <xsl:function name="m:complexTypes-with-simpleContent" as="document-node()">
        <xsl:param name="schemas" as="document-node()"/>
        
        <xsl:variable name="updated-schemas">
            <schemas>
                <xsl:apply-templates select="$schemas/schemas/xs:schema" mode="complexTypes-with-simpleContent"/>
            </schemas>
        </xsl:variable>

        <xsl:sequence select="$updated-schemas"/>

    </xsl:function>

    <xsl:template match="xs:schema" mode="complexTypes-with-simpleContent">
        <xsl:copy>
            <xsl:copy-of select="@*"/>
            <xsl:apply-templates mode="complexTypes-with-simpleContent"/>
        </xsl:copy>

    </xsl:template>
    
    <xsl:template match="xs:complexType[xs:simpleContent/xs:extension[namespace-uri-from-QName(resolve-QName(@base, ..)) eq 'http://www.w3.org/2001/XMLSchema'][not(xs:attribute)][not(xs:attributeGroup)][not(xs:anyAttribute)]]" mode="complexTypes-with-simpleContent">
        
        <xsl:sequence select="." />
        
        <!--xs:simpleType>
            <xsl:copy-of select="@*"/>
            <xs:restriction base="{xs:simpleContent/xs:extension/@base}">
                <xsl:sequence select="xs:simpleContent/xs:extension/node()" />
            </xs:restriction>
        </xs:simpleType-->
        
    </xsl:template>
    
    <xsl:template match="xs:complexType[xs:simpleContent/xs:extension[namespace-uri-from-QName(resolve-QName(@base, ..)) ne 'http://www.w3.org/2001/XMLSchema'][not(xs:attribute)][not(xs:attributeGroup)][not(xs:anyAttribute)]]" mode="complexTypes-with-simpleContent">
        
        <xsl:sequence select="." />
        
        <!--xsl:variable name="referenced-simpleType">
            <xsl:apply-templates select="key('ids', xs:simpleContent/xs:extension/@idref)" mode="complexTypes-with-simpleContent" />
        </xsl:variable>
        
        <xsl:choose>
            <xsl:when test="$referenced-simpleType[xs:simpleType]">
                <xs:simpleType>
                    <xsl:copy-of select="@*"/>
                    <xsl:copy-of select="$referenced-simpleType/xs:simpleType/node()" />
                </xs:simpleType>
            </xsl:when>
            <xsl:otherwise>
                <xsl:sequence select="." />
            </xsl:otherwise>
        </xsl:choose-->
        
    </xsl:template>
    
    
    <xsl:template match="node()" mode="complexTypes-with-simpleContent">
        <xsl:copy>
            <xsl:copy-of select="@*"/>
            <xsl:apply-templates mode="complexTypes-with-simpleContent"/>
        </xsl:copy>
        
    </xsl:template>

</xsl:stylesheet>
