<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
                xmlns:q="http://www.schema-library.org/show/QName/items" 
                xmlns:xs="http://www.w3.org/2001/XMLSchema" 
                exclude-result-prefixes="#all" 
                version="2.0">
    
    <xsl:key name="ids" match="*[@id]" use="@id"/>
    
    <xsl:function name="q:get-QName-elements-and-attributes" as="document-node()">
        <xsl:param name="schemas" as="document-node()"/>

        <xsl:variable name="updated-schemas">
            <schemas>
                <xsl:apply-templates select="$schemas/schemas/xs:schema[1]" mode="get-QName-elements-and-attributes"/>
            </schemas>
        </xsl:variable>

        <xsl:sequence select="$updated-schemas"/>

    </xsl:function>
    
    <xsl:template match="xs:schema" mode="get-QName-elements-and-attributes">
        
        <xsl:apply-templates select="xs:element" mode="get-QName-elements-and-attributes" />
        
    </xsl:template>

    <xsl:template match="*" mode="get-QName-elements-and-attributes">

        <xsl:apply-templates mode="get-QName-elements-and-attributes" />

    </xsl:template>
    
    <xsl:template match="xs:element[xs:simpleType/xs:restriction/@base]
                                   [namespace-uri-from-QName(resolve-QName(xs:simpleType/xs:restriction/@base, .)) eq 'http://www.w3.org/2001/XMLSchema']
                                   [local-name-from-QName(resolve-QName(xs:simpleType/xs:restriction/@base, .)) eq 'QName']" mode="get-QName-elements-and-attributes">
        <xsl:variable name="here" select="." />
        <xsl:variable name="xpath" select="string-join(q:XPath-to-item(..), '')" />
        <xsl:variable name="targetNamespace" select="@targetNamespace" />
        <xsl:value-of select='concat($xpath, "/*[local-name()=&apos;", @name, "&apos;][namespace-uri()=&apos;", $targetNamespace, "&apos;]", "/text()")' />
        <xsl:text>
</xsl:text> 
        <xsl:for-each select="tokenize(normalize-space(@substitutable-by-these-elements), '\s+')">
            <xsl:variable name="idref" select="." />
            <xsl:for-each select="$here">
                <xsl:variable name="element" select="key('ids', $idref)" />
                <xsl:value-of select='concat($xpath, "/*[local-name()=&apos;", $element/@name, "&apos;][namespace-uri()=&apos;", $element/$targetNamespace, "&apos;]", "/text()")' />
                <xsl:text>
</xsl:text> 
            </xsl:for-each>
        </xsl:for-each>
        
    </xsl:template>
    
    <xsl:template match="xs:element[@type]
                                   [namespace-uri-from-QName(resolve-QName(@type, .)) eq 'http://www.w3.org/2001/XMLSchema']
                                   [local-name-from-QName(resolve-QName(@type, .)) eq 'QName']" mode="get-QName-elements-and-attributes">
        <xsl:variable name="here" select="." />
        <xsl:variable name="xpath" select="string-join(q:XPath-to-item(..), '')" />
        <xsl:variable name="targetNamespace" select="@targetNamespace" />
        <xsl:value-of select='concat($xpath, "/*[local-name()=&apos;", @name, "&apos;][namespace-uri()=&apos;", $targetNamespace, "&apos;]", "/text()")' />
        <xsl:text>
</xsl:text> 
        <xsl:for-each select="tokenize(normalize-space(@substitutable-by-these-elements), '\s+')">
            <xsl:variable name="idref" select="." />
            <xsl:for-each select="$here">
                <xsl:variable name="element" select="key('ids', $idref)" />
                <xsl:value-of select='concat($xpath, "/*[local-name()=&apos;", $element/@name, "&apos;][namespace-uri()=&apos;", $element/$targetNamespace, "&apos;]", "/text()")' />
                <xsl:text>
</xsl:text> 
            </xsl:for-each>
        </xsl:for-each>
        
    </xsl:template>
    
    <xsl:template match="xs:element[xs:simpleType/xs:list/@itemType]
                                   [namespace-uri-from-QName(resolve-QName(xs:simpleType/xs:list/@itemType, .)) eq 'http://www.w3.org/2001/XMLSchema']
                                   [local-name-from-QName(resolve-QName(xs:simpleType/xs:list/@itemType, .)) eq 'QName']" mode="get-QName-elements-and-attributes">
        <xsl:variable name="here" select="." />
        <xsl:variable name="xpath" select="string-join(q:XPath-to-item(..), '')" />
        <xsl:variable name="targetNamespace" select="@targetNamespace" />
        <xsl:value-of select='concat($xpath, "/*[local-name()=&apos;", @name, "&apos;][namespace-uri()=&apos;", $targetNamespace, "&apos;]", "/text()")' />
        <xsl:text>
</xsl:text> 
        <xsl:for-each select="tokenize(normalize-space(@substitutable-by-these-elements), '\s+')">
            <xsl:variable name="idref" select="." />
            <xsl:for-each select="$here">
                <xsl:variable name="element" select="key('ids', $idref)" />
                <xsl:value-of select='concat($xpath, "/*[local-name()=&apos;", $element/@name, "&apos;][namespace-uri()=&apos;", $element/$targetNamespace, "&apos;]", "/text()")' />
                <xsl:text>
</xsl:text> 
            </xsl:for-each>
        </xsl:for-each>
        
    </xsl:template>
    
    <xsl:template match="xs:element[xs:simpleType/xs:union]" mode="get-QName-elements-and-attributes">
        <xsl:variable name="here" select="." />
        
        <xsl:variable name="QName-is-in-union" as="xs:boolean*">
            <xsl:for-each select="xs:simpleType/xs:union//(@base, @itemType)">
                <xsl:if test="(namespace-uri-from-QName(resolve-QName(., ..)) eq 'http://www.w3.org/2001/XMLSchema') and
                              (local-name-from-QName(resolve-QName(., ..)) eq 'QName')">
                    <xsl:value-of select="true()" />
                </xsl:if>
            </xsl:for-each>
        </xsl:variable>
        <xsl:if test="not(empty($QName-is-in-union))">
            <xsl:variable name="here" select="." />
            <xsl:variable name="xpath" select="string-join(q:XPath-to-item(..), '')" />
            <xsl:variable name="targetNamespace" select="@targetNamespace" />
            <xsl:value-of select='concat($xpath, "/*[local-name()=&apos;", @name, "&apos;][namespace-uri()=&apos;", $targetNamespace, "&apos;]", "/text()")' />
            <xsl:text>
</xsl:text> 
            <xsl:for-each select="tokenize(normalize-space(@substitutable-by-these-elements), '\s+')">
                <xsl:variable name="idref" select="." />
                <xsl:for-each select="$here">
                    <xsl:variable name="element" select="key('ids', $idref)" />
                    <xsl:value-of select='concat($xpath, "/*[local-name()=&apos;", $element/@name, "&apos;][namespace-uri()=&apos;", $element/$targetNamespace, "&apos;]", "/text()")' />
                    <xsl:text>
</xsl:text> 
                </xsl:for-each>
            </xsl:for-each>
        </xsl:if>
        
        
    </xsl:template>
    
    <xsl:template match="xs:element[xs:complexType/xs:simpleContent/xs:extension/@base]
                                   [namespace-uri-from-QName(resolve-QName(xs:complexType/xs:simpleContent/xs:extension/@base, .)) eq 'http://www.w3.org/2001/XMLSchema']
                                   [local-name-from-QName(resolve-QName(xs:complexType/xs:simpleContent/xs:extension/@base, .)) eq 'QName']" mode="get-QName-elements-and-attributes">
        
        <xsl:variable name="here" select="." />
        <xsl:variable name="xpath" select="string-join(q:XPath-to-item(..), '')" />
        <xsl:variable name="targetNamespace" select="@targetNamespace" />
        <xsl:value-of select='concat($xpath, "/*[local-name()=&apos;", @name, "&apos;][namespace-uri()=&apos;", $targetNamespace, "&apos;]", "/text()")' />
        <xsl:text>
</xsl:text> 
        <xsl:for-each select="tokenize(normalize-space(@substitutable-by-these-elements), '\s+')">
            <xsl:variable name="idref" select="." />
            <xsl:for-each select="$here">
                <xsl:variable name="element" select="key('ids', $idref)" />
                <xsl:value-of select='concat($xpath, "/*[local-name()=&apos;", $element/@name, "&apos;][namespace-uri()=&apos;", $element/$targetNamespace, "&apos;]", "/text()")' />
                <xsl:text>
</xsl:text> 
            </xsl:for-each>
        </xsl:for-each>
        
        <xsl:apply-templates select="xs:complexType/xs:simpleContent/xs:extension/xs:attribute" mode="get-QName-elements-and-attributes" />
        
    </xsl:template>
    
    <xsl:template match="xs:element[xs:complexType/xs:simpleContent/xs:restriction/@base]" mode="get-QName-elements-and-attributes">
        
        <xsl:variable name="complexType" select="key('ids', xs:complexType/xs:simpleContent/xs:restriction/@idref)" />
        
        <xsl:if test="$complexType[xs:simpleContent/xs:extension/@base]
                                  [namespace-uri-from-QName(resolve-QName(xs:simpleContent/xs:extension/@base, .)) eq 'http://www.w3.org/2001/XMLSchema']
                                  [local-name-from-QName(resolve-QName(xs:simpleContent/xs:extension/@base, .)) eq 'QName']">
            <xsl:variable name="here" select="." />
            <xsl:variable name="xpath" select="string-join(q:XPath-to-item(..), '')" />
            <xsl:variable name="targetNamespace" select="@targetNamespace" />
            <xsl:value-of select='concat($xpath, "/*[local-name()=&apos;", @name, "&apos;][namespace-uri()=&apos;", $targetNamespace, "&apos;]", "/text()")' />
            <xsl:text>
</xsl:text> 
            <xsl:for-each select="tokenize(normalize-space(@substitutable-by-these-elements), '\s+')">
                <xsl:variable name="idref" select="." />
                <xsl:for-each select="$here">
                    <xsl:variable name="element" select="key('ids', $idref)" />
                    <xsl:value-of select='concat($xpath, "/*[local-name()=&apos;", $element/@name, "&apos;][namespace-uri()=&apos;", $element/$targetNamespace, "&apos;]", "/text()")' />
                    <xsl:text>
</xsl:text> 
                </xsl:for-each>
            </xsl:for-each>
            
            <xsl:apply-templates select="xs:complexType/xs:simpleContent/xs:restriction/xs:attribute" mode="get-QName-elements-and-attributes" />
        </xsl:if>
           
    </xsl:template>
    
    <xsl:template match="xs:attribute[xs:simpleType/xs:restriction/@base]
                                     [namespace-uri-from-QName(resolve-QName(xs:simpleType/xs:restriction/@base, .)) eq 'http://www.w3.org/2001/XMLSchema']
                                     [local-name-from-QName(resolve-QName(xs:simpleType/xs:restriction/@base, .)) eq 'QName']" mode="get-QName-elements-and-attributes">
        <xsl:value-of select="q:XPath-to-item(..)" />
        <xsl:text>/@</xsl:text>
        <xsl:value-of select="@name" />
        <xsl:text>
</xsl:text> 
        
    </xsl:template>
    
    <xsl:template match="xs:attribute[@type]
                                     [namespace-uri-from-QName(resolve-QName(@type, .)) eq 'http://www.w3.org/2001/XMLSchema']
                                     [local-name-from-QName(resolve-QName(@type, .)) eq 'QName']" mode="get-QName-elements-and-attributes">
        <xsl:value-of select="q:XPath-to-item(..)" />
        <xsl:text>/@</xsl:text>
        <xsl:value-of select="@name" />
        <xsl:text> 
</xsl:text>   
    </xsl:template>
    
    <xsl:template match="xs:attribute[xs:simpleType/xs:list/@itemType]
                                     [namespace-uri-from-QName(resolve-QName(xs:simpleType/xs:list/@itemType, .)) eq 'http://www.w3.org/2001/XMLSchema']
                                     [local-name-from-QName(resolve-QName(xs:simpleType/xs:list/@itemType, .)) eq 'QName']" mode="get-QName-elements-and-attributes">
        <xsl:value-of select="q:XPath-to-item(..)" />
        <xsl:text>/@</xsl:text>
        <xsl:value-of select="@name" />
        <xsl:text> 
</xsl:text>   
        
    </xsl:template>
    
    <xsl:template match="xs:attribute[xs:simpleType/xs:union]" mode="get-QName-elements-and-attributes">
        <xsl:variable name="here" select="." />
        
        <xsl:variable name="QName-is-in-union" as="xs:boolean*">
            <xsl:for-each select="xs:simpleType/xs:union//(@base, @itemType)">
                <xsl:if test="(namespace-uri-from-QName(resolve-QName(., ..)) eq 'http://www.w3.org/2001/XMLSchema') and
                    (local-name-from-QName(resolve-QName(., ..)) eq 'QName')">
                    <xsl:value-of select="true()" />
                </xsl:if>
            </xsl:for-each>
        </xsl:variable>
        <xsl:if test="not(empty($QName-is-in-union))">
            <xsl:value-of select="q:XPath-to-item(..)" />
            <xsl:text>/@</xsl:text>
            <xsl:value-of select="@name" />
            <xsl:text> 
</xsl:text>   
        </xsl:if>
        
    </xsl:template>
    
    <xsl:template match="text()" mode="get-QName-elements-and-attributes" />

    <xsl:function name="q:XPath-to-item">
        <xsl:param name="item" as="element()" />  
        
        <xsl:for-each select="$item/ancestor-or-self::*">
            <xsl:choose>
                <xsl:when test="self::xs:element">
                    <xsl:text>/*[local-name()='</xsl:text>
                    <xsl:value-of select="@name"/>
                    <xsl:text>']</xsl:text>
                    <xsl:text>[namespace-uri()='</xsl:text>
                    <xsl:value-of select="@targetNamespace"/>
                    <xsl:text>']</xsl:text>
                    <!--xsl:variable name="name" select="name()"/>
                    <xsl:choose>
                        <xsl:when test="count(preceding-sibling::*[name() = $name]) &gt; 0">
                            <xsl:text>[</xsl:text><xsl:value-of select="count(preceding-sibling::*[name() = $name]) + 1"/><xsl:text>]</xsl:text>
                        </xsl:when>
                    </xsl:choose-->
                </xsl:when>
            </xsl:choose>
        </xsl:for-each>
        
    </xsl:function>

</xsl:stylesheet>
