<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
                xmlns:s="http://www.schema-library.org/show/elements/type" 
                xmlns:s2="http://www.schema-library.org/show/substitutionGroup/elements"
                xmlns:rw="http://www.schema-library.org/rewrite"
                xmlns:red="http://www.schema-library.org/reduce" 
                xmlns:sic="http://www.schema-library.org/schema-in-context" 
                xmlns:h="http://www.schema-library.org/input-schema-and-its-dependency-hierarchy"
                xmlns:removed="http://www.remove.org" 
                xmlns:xs="http://www.w3.org/2001/XMLSchema" 
                exclude-result-prefixes="#all" 
                version="2.0">

    <!-- 
        A complexType may restrict another. This program merges the
        the base type and the child type.  Thus, the child
        type is standalone (no longer dependent on a base type).
        
        Author: Roger L. Costello
    -->
    
    <xsl:key name="ids" match="*[@_id]" use="@_id"/>

    <xsl:function name="s:inline-complexContent-derive-by-restriction" as="document-node()">
        <xsl:param name="schemas" as="document-node()"/>

        <xsl:variable name="updated-schemas">
            <schemas>
                <xsl:apply-templates select="$schemas/schemas/xs:schema" mode="inline-derive-by-restriction"/>
            </schemas>
        </xsl:variable>

        <xsl:sequence select="$updated-schemas"/>

    </xsl:function>

    <xsl:template match="xs:schema" mode="inline-derive-by-restriction">

        <xsl:copy>
            <xsl:copy-of select="@*"/>
            <xsl:apply-templates mode="inline-derive-by-restriction" />
        </xsl:copy>

    </xsl:template>
    
    <xsl:template match="xs:complexType[xs:complexContent/xs:restriction]" mode="inline-derive-by-restriction">
        
        <xsl:copy>
            <xsl:copy-of select="@*"/>
            <!-- The attribute, mixed, can be on complexContent so I
                 move it up onto the complexType element. -->
            <xsl:copy-of select="xs:complexContent/@mixed"/>
            <xsl:apply-templates select="xs:complexContent/xs:restriction/(xs:sequence | xs:choice | xs:all)" mode="inline-derive-by-restriction" />
            
            <!-- Need to union the attributes in the base type with the attributes in the derived type -->
            <xsl:variable name="attribute-names" select="xs:complexContent/xs:restriction/xs:attribute/@name" />
            <xsl:for-each select="xs:complexContent/xs:restriction/key('ids', @idref)/xs:attribute[not(@name = $attribute-names)] ">
                <xsl:sequence select="." />
            </xsl:for-each>
            <xsl:for-each select="xs:complexContent/xs:restriction/xs:attribute">
                <xsl:sequence select="."/>
            </xsl:for-each>
            <xsl:choose>
                <xsl:when test="xs:complexContent/xs:restriction/xs:anyAttribute">
                    <xsl:sequence select="xs:complexContent/xs:restriction/xs:anyAttribute"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:sequence select="xs:complexContent/xs:restriction/key('ids', @idref)/xs:anyAttribute"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:copy>
        
    </xsl:template>
    
    <xsl:template match="node()" mode="inline-derive-by-restriction">
        
        <xsl:copy>
            <xsl:copy-of select="@*"/>
            <xsl:apply-templates mode="inline-derive-by-restriction" />
        </xsl:copy>
        
    </xsl:template>

</xsl:stylesheet>
