<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
                xmlns:s="http://www.schema-library.org/show/elements/type" 
                xmlns:xs="http://www.w3.org/2001/XMLSchema" 
                exclude-result-prefixes="#all" 
                version="2.0">

    <!-- 
        Suppose an element declaration has a ref attribute. This program
        will fetch the element being ref'ed and replace the ref'ing 
        element declaration with the ref'ed element (of course, it retains
        minOccurs and maxOccurs from the ref'ing element). Ditto for attribute
        declarations containing a ref attribute, group elements containing
        a ref attribute, and attributeGroup elements containing a ref
        attribute.
        
        Author: Roger L. Costello
    -->
    
    <xsl:key name="ids" match="*[@_id]" use="@_id"/>

    <xsl:function name="s:inline-ref" as="document-node()">
        <xsl:param name="schemas" as="document-node()"/>
        
        <xsl:variable name="result">
            <schemas>
                <xsl:for-each select="$schemas/schemas/xs:schema">
                    <xsl:message>PROCESSING SCHEMA #<xsl:value-of select="position()" /></xsl:message>
                    <xsl:apply-templates select="." mode="inline-ref" />
                </xsl:for-each>
            </schemas>
        </xsl:variable>
        
        <xsl:sequence select="$result" />

    </xsl:function>
    
    
    <xsl:template match="xs:schema" mode="inline-ref">
        
        <xsl:copy>
            <xsl:copy-of select="@*"/>
            <xsl:apply-templates mode="inline-ref">
                <xsl:with-param name="idrefs-already-seen" select="()" />
            </xsl:apply-templates>
        </xsl:copy>
        
    </xsl:template>
    
    
    <xsl:template match="xs:element[@ref]" mode="inline-ref">
        <xsl:param name="idrefs-already-seen" />
        
        <xsl:choose>
            <xsl:when test="@idref = $idrefs-already-seen">
                <xsl:sequence select="." />
            </xsl:when>
            <xsl:otherwise>
                <xsl:variable name="here" select="." />
                <xsl:variable name="refed-element" select="key('ids', @idref)"/>
                <xsl:copy>
                    <xsl:copy-of select="$refed-element/(@* except @_id)" />  
                    <xsl:copy-of select="@* except (@ref, @idref)" />      
                    <xsl:sequence select="child::node()" />
                    <xsl:apply-templates select="$refed-element/child::node()" mode="inline-ref">
                        <xsl:with-param name="idrefs-already-seen" select="($idrefs-already-seen, $here/@idref)" />
                    </xsl:apply-templates>
                </xsl:copy>
            </xsl:otherwise>
        </xsl:choose>
        
    </xsl:template>
    
    
    <xsl:template match="xs:attribute[@ref]" mode="inline-ref">
        <xsl:param name="idrefs-already-seen" />
        
        <xsl:choose>
            <xsl:when test="@idref = $idrefs-already-seen">
                <xsl:sequence select="." />
            </xsl:when>
            <xsl:otherwise>
                <xsl:variable name="here" select="." />
                <xsl:variable name="refed-attribute" select="key('ids', @idref)"/>
                <xsl:copy>
                    <xsl:copy-of select="$refed-attribute/(@* except @_id)" />   <!-- The global attribute has an id attribute. Don't put it on the local attribute; otherwise there would be 
                                                                                     multiple items with the same id. -->
                    <xsl:copy-of select="@* except (@ref, @idref)" />           <!-- Since the ref attribute is removed, the idref must also be removed. --> 
                    <xsl:sequence select="child::node()" />
                    <xsl:apply-templates select="$refed-attribute/child::node()" mode="inline-ref">
                        <xsl:with-param name="idrefs-already-seen" select="($idrefs-already-seen, $here/@idref)" />
                    </xsl:apply-templates>
                </xsl:copy>
            </xsl:otherwise>
        </xsl:choose>
        
    </xsl:template>
    
    
    <xsl:template match="xs:group[@ref]" mode="inline-ref">
        <xsl:param name="idrefs-already-seen" />
        
        <xsl:variable name="here" select="." />
        
        <xsl:variable name="refed-group" select="key('ids', @idref)"/>
        
        <xsl:apply-templates select="$refed-group/node()" mode="inline-ref">
            <xsl:with-param name="idrefs-already-seen" select="($idrefs-already-seen, $here/@idref)" />
        </xsl:apply-templates>
        
    </xsl:template>
    
    
    <xsl:template match="xs:attributeGroup[@ref]" mode="inline-ref">
        <xsl:param name="idrefs-already-seen" />
        
        <xsl:variable name="here" select="." />
        
        <xsl:variable name="refed-attributeGroup" select="key('ids', @idref)"/>
        
        <xsl:apply-templates select="$refed-attributeGroup/node()" mode="inline-ref">
            <xsl:with-param name="idrefs-already-seen" select="($idrefs-already-seen, $here/@idref)" />
        </xsl:apply-templates>
        
    </xsl:template>
    
    
    <xsl:template match="node()" mode="inline-ref">
        <xsl:param name="idrefs-already-seen" />
        
        <xsl:copy>
            <xsl:copy-of select="@*"/>
            <xsl:apply-templates mode="inline-ref">
                <xsl:with-param name="idrefs-already-seen" select="$idrefs-already-seen" />
            </xsl:apply-templates>
        </xsl:copy>
        
    </xsl:template>

</xsl:stylesheet>
