<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
                xmlns:s="http://www.schema-library.org/show/elements/type" 
                xmlns:xs="http://www.w3.org/2001/XMLSchema" 
                exclude-result-prefixes="#all" 
                version="2.0">

    <!-- 
        Suppose an element declaration has a type attribute. This program
        fetches the item referenced by the type attribute and inlines the 
        item within the element (and removes the type attribute). Ditto for
        attribute declarations with a type attribute.
        
        Author: Roger L. Costello
    -->
    
    <xsl:key name="ids" match="*[@_id]" use="@_id"/>

    <xsl:function name="s:inline-types" as="document-node()">
        <xsl:param name="schemas" as="document-node()"/>

        <xsl:variable name="updated-schemas">
            <schemas>
                <xsl:apply-templates select="$schemas/schemas/xs:schema" mode="inline-types"/>
            </schemas>
        </xsl:variable>

        <xsl:sequence select="$updated-schemas"/>

    </xsl:function>

    <xsl:template match="xs:schema" mode="inline-types">
        <xsl:copy>
            <xsl:copy-of select="@*"/>
            <xsl:apply-templates mode="inline-types" />
        </xsl:copy>
    </xsl:template>
    
    
    <!-- This template handles the case of an element A that references a complexType B
         which has the element A in its content. That is, this template handles 
         infinitely recursive references. -->
    
    <xsl:template match="xs:element[@type][namespace-uri-from-QName(resolve-QName(@type, .)) ne 'http://www.w3.org/2001/XMLSchema'][@idref = ancestor::xs:complexType/@_id]" mode="inline-types">
        <xsl:sequence select="." />
    </xsl:template>
    
    <xsl:template match="xs:element[@type][namespace-uri-from-QName(resolve-QName(@type, .)) ne 'http://www.w3.org/2001/XMLSchema'][not(@idref = ancestor::xs:complexType/@_id)]" mode="inline-types">
        <xsl:copy>
            <xsl:copy-of select="@* except (@type, @idref)"/>
            <xsl:sequence select="node()" />
            <xsl:variable name="type" select="key('ids', @idref)"/>
            <xsl:variable name="processed-type" as="element()">
                <xsl:apply-templates select="$type" mode="inline-types" />
            </xsl:variable>
            <xsl:for-each select="$processed-type">
                <xsl:copy>
                    <xsl:copy-of select="@* except (@name, @_id)"/>  <!-- The global type has an id attribute. Don't put it on the local element; otherwise there would be 
                                                                         multiple elements with the same id. -->
                    <xsl:sequence select="*" />
                </xsl:copy>
            </xsl:for-each>
        </xsl:copy>
    </xsl:template>
    
    <xsl:template match="xs:attribute[@type][namespace-uri-from-QName(resolve-QName(@type, .)) ne 'http://www.w3.org/2001/XMLSchema']" mode="inline-types">
        <xsl:variable name="type" select="key('ids', @idref)"/>
        
        <xsl:copy>
            <xsl:copy-of select="@* except (@type, @idref)"/>
            <xsl:sequence select="node()" />
            <xsl:for-each select="$type">
                <xsl:copy>
                    <xsl:copy-of select="@* except (@name, @_id)"/>
                    <xsl:sequence select="*" />
                </xsl:copy>
            </xsl:for-each>
        </xsl:copy>
    </xsl:template>
    
    <xsl:template match="node()" mode="inline-types">
        
        <xsl:copy>
            <xsl:copy-of select="@*"/>
            <xsl:apply-templates mode="inline-types" />
        </xsl:copy>
        
    </xsl:template>

</xsl:stylesheet>
