<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
                xmlns:s="http://www.schema-library.org/show/elements/type" 
                xmlns:xs="http://www.w3.org/2001/XMLSchema" 
                exclude-result-prefixes="#all" 
                version="2.0">

    <!-- 
        A simpleType may be at the end of a long simpleType chain
        (simpleType A restricts simpleType B, which restricts C and so on).
        This program merges the chain down into a single simpleType.
        
        Author: Roger L. Costello
    -->
    
    <xsl:key name="ids" match="*[@_id]" use="@_id"/>

    <xsl:function name="s:reduce-simpleTypes" as="document-node()">
        <xsl:param name="schemas" as="document-node()"/>

        <xsl:variable name="updated-schemas">
            <schemas>
                <xsl:apply-templates select="$schemas/schemas/xs:schema" mode="reduce-simpleTypes"/>
            </schemas>
        </xsl:variable>

        <xsl:sequence select="$updated-schemas"/>

    </xsl:function>

    <xsl:template match="xs:schema" mode="reduce-simpleTypes">

        <xsl:copy>
            <xsl:copy-of select="@*"/>
            <xsl:apply-templates mode="reduce-simpleTypes" />
        </xsl:copy>

    </xsl:template>
    
    <xsl:template match="xs:simpleType[xs:restriction/@base][namespace-uri-from-QName(resolve-QName(xs:restriction/@base, .)) ne 'http://www.w3.org/2001/XMLSchema']" mode="reduce-simpleTypes">
        
        <xsl:variable name="child-simpleType" select="."/>
        
        <xsl:for-each select="xs:restriction">
            <xsl:variable name="parent-simpleType" select="key('ids', @idref)"/>
            <xsl:variable name="updated-parent-simpleType" as="element(xs:simpleType)">
                <xsl:apply-templates select="$parent-simpleType" mode="reduce-simpleTypes" />
            </xsl:variable>
            <xsl:sequence select="s:simpleType-reduce($updated-parent-simpleType, $child-simpleType)" />
        </xsl:for-each>
        
    </xsl:template>
    
    <xsl:template match="xs:simpleType[xs:list/@itemType][namespace-uri-from-QName(resolve-QName(xs:list/@itemType, .)) ne 'http://www.w3.org/2001/XMLSchema']" mode="reduce-simpleTypes">
        
        <xsl:variable name="child-simpleType" select="."/>
        
        <xsl:for-each select="xs:list">
            <xsl:variable name="parent-simpleType" select="key('ids', @idref)"/>
            <xsl:variable name="updated-parent-simpleType" as="element(xs:simpleType)">
                <xsl:apply-templates select="$parent-simpleType" mode="reduce-simpleTypes" />
            </xsl:variable>
            <xsl:sequence select="s:simpleType-reduce($updated-parent-simpleType, $child-simpleType)" />
        </xsl:for-each>
        
    </xsl:template>
    
    <xsl:template match="xs:simpleType[xs:union/@memberTypes]" mode="reduce-simpleTypes">

        <xsl:variable name="child-simpleType" select="."/>
        
        <xsl:for-each select="$child-simpleType">
            <xsl:copy>
                <xsl:copy-of select="@*"/>
                <xsl:sequence select="node()[not(self::xs:union)]" />
                <xsl:for-each select="xs:union">
                    <xsl:copy>
                        <xsl:copy-of select="@* except (@memberTypes, @idrefs)"/>
                        <xsl:for-each select="tokenize(normalize-space(@idrefs), '\s')">
                            <xsl:variable name="idref" select="." />
                            <xsl:for-each select="$child-simpleType/xs:union">
                                <xsl:variable name="parent-simpleType" select="key('ids', $idref)"/>
                                <xsl:variable name="updated-parent-simpleType" as="element(xs:simpleType)">
                                    <xsl:apply-templates select="$parent-simpleType" mode="reduce-simpleTypes" />
                                </xsl:variable>
                                <xsl:sequence select="s:sans-name-and-id($updated-parent-simpleType)" />
                            </xsl:for-each>
                        </xsl:for-each>
                        <xsl:for-each select="node()">
                            <xsl:sequence select="." />
                        </xsl:for-each>
                    </xsl:copy>
                </xsl:for-each>
            </xsl:copy>
        </xsl:for-each>
        
    </xsl:template>
    
    <xsl:template match="node()" mode="reduce-simpleTypes">
        <xsl:copy>
            <xsl:copy-of select="@*"/>
            <xsl:apply-templates mode="reduce-simpleTypes" />
        </xsl:copy>
    </xsl:template>
    
    <!-- 
        simpleType-reduce :: SimpleType -> Schema-in-Context -> SimpleType
    -->
    <xsl:function name="s:simpleType-reduce">
        <xsl:param name="parent-simpleType" />
        <xsl:param name="child-simpleType" />
        
        <xsl:choose>
            
            <xsl:when test="exists($parent-simpleType/xs:restriction) and exists($child-simpleType/xs:restriction)">
                <xsl:for-each select="$child-simpleType">
                    <xsl:copy>
                        <xsl:copy-of select="@*"/>
                        <xsl:sequence select="node()[not(self::xs:restriction)]" />
                        <xsl:for-each select="$child-simpleType/xs:restriction">
                            <xsl:copy>
                                <xsl:copy-of select="@* except @base"/>
                                <xsl:copy-of select="$parent-simpleType/xs:restriction/@base"/>
                                <xsl:sequence select="s:merge($parent-simpleType/xs:restriction/(xs:* except xs:annotation), $child-simpleType/xs:restriction/(xs:* except xs:annotation))" />
                            </xsl:copy>
                        </xsl:for-each>
                    </xsl:copy>
                </xsl:for-each>
            </xsl:when>
            
            <xsl:when test="exists($child-simpleType/xs:list/@itemType)">
                <xsl:for-each select="$child-simpleType">
                    <xsl:copy>
                        <xsl:copy-of select="@*"/>
                        <xsl:sequence select="node()[not(self::xs:list)]" />
                        <xsl:for-each select="$child-simpleType/xs:list">
                            <xsl:copy>
                                <xsl:copy-of select="@* except @itemType"/>
                                <xsl:sequence select="s:sans-name-and-id($parent-simpleType)" />
                            </xsl:copy>
                        </xsl:for-each>
                    </xsl:copy>
                </xsl:for-each>
            </xsl:when>
            
            <xsl:when test="exists($parent-simpleType/xs:list/xs:simpleType) and exists($child-simpleType/xs:restriction)">
                <xsl:for-each select="$child-simpleType">
                    <xsl:copy>
                        <xsl:copy-of select="@*"/>
                        <xsl:sequence select="node()[not(self::xs:restriction)]" />
                        <xsl:for-each select="xs:restriction">
                            <xsl:copy>
                                <xsl:copy-of select="@* except @base"/>
                                <xsl:sequence select="s:sans-name-and-id($parent-simpleType)" />
                                <xsl:sequence select="xs:* except xs:annotation" />
                            </xsl:copy>
                        </xsl:for-each>
                    </xsl:copy>
                </xsl:for-each>
            </xsl:when>
            
            <!-- Added this on June 9, 2012 -->
            
            <xsl:when test="exists($parent-simpleType/xs:list/@itemType) and exists($child-simpleType/xs:restriction)">
                <xsl:for-each select="$child-simpleType">
                    <xsl:copy>
                        <xsl:copy-of select="@*"/>
                        <xsl:sequence select="node()[not(self::xs:restriction)]" />
                        <xsl:for-each select="xs:restriction">
                            <xsl:copy>
                                <xsl:copy-of select="@* except @base"/>
                                <xsl:sequence select="s:sans-name-and-id($parent-simpleType)" />
                                <xsl:sequence select="xs:* except xs:annotation" />
                            </xsl:copy>
                        </xsl:for-each>
                    </xsl:copy>
                </xsl:for-each>
            </xsl:when>
            
            <!-- Added this on June 9, 2012 -->
            
            <xsl:when test="exists($parent-simpleType/xs:union) and exists($child-simpleType/xs:restriction)">
                <xsl:for-each select="$child-simpleType">
                    <xsl:copy>
                        <xsl:copy-of select="@*"/>
                        <xsl:sequence select="node()[not(self::xs:restriction)]" />
                        <xsl:for-each select="$parent-simpleType/xs:union">
                            <xsl:copy>
                                <xsl:copy-of select="@*"/>
                                <xsl:sequence select="xs:* except xs:annotation" />
                            </xsl:copy>
                        </xsl:for-each>
                    </xsl:copy>
                </xsl:for-each>
            </xsl:when>
            
            <!-- 
                
                If none of the above patterns match then either:
                (a) The simpleTypes are invalid, or
                (b) I've missed a case
                
            -->
            
            <xsl:otherwise>
                <xsl:message terminate="yes">
                    [Error] reduce.xsd, function s:simpleType-reduce The simpleTypes provided do not match any of the patterns.
                    parent-simpleType = <xsl:sequence select="$parent-simpleType" />
                    child-simpleType = <xsl:sequence select="$child-simpleType" />
                </xsl:message>
            </xsl:otherwise>
            
        </xsl:choose>
        
    </xsl:function> 
    
    <!-- 
        merge :: [Facet] -> [Facet] -> [Facet]
    -->
    <xsl:function name="s:merge">
        <xsl:param name="parent-facets" />
        <xsl:param name="child-facets" />
        
        <xsl:sequence select="s:AND-facets(for $i in $parent-facets[self::xs:length] return $i, for $i in $child-facets[self::xs:length] return $i)" />
        
        <xsl:sequence select="s:AND-facets(for $i in $parent-facets[self::xs:minLength] return $i, for $i in $child-facets[self::xs:minLength] return $i)" />
        <xsl:sequence select="s:AND-facets(for $i in $parent-facets[self::xs:maxLength] return $i, for $i in $child-facets[self::xs:maxLength] return $i)" />
        
        <xsl:choose>
            <xsl:when test="exists(for $i in $parent-facets[self::xs:minInclusive] return $i) and exists(for $i in $child-facets[self::xs:minExclusive] return $i)">
                <xsl:sequence select="for $i in $child-facets[self::xs:minExclusive] return $i" />
            </xsl:when>
            <xsl:when test="exists(for $i in $parent-facets[self::xs:minExclusive] return $i) and exists(for $i in $child-facets[self::xs:minInclusive] return $i)">
                <xsl:sequence select="for $i in $child-facets[self::xs:minInclusive] return $i" />
            </xsl:when>
            <xsl:otherwise>
                <xsl:sequence select="s:AND-facets(for $i in $parent-facets[self::xs:minInclusive] return $i, for $i in $child-facets[self::xs:minInclusive] return $i)" />
                <xsl:sequence select="s:AND-facets(for $i in $parent-facets[self::xs:minExclusive] return $i, for $i in $child-facets[self::xs:minExclusive] return $i)" />
            </xsl:otherwise>
        </xsl:choose>
        
        <xsl:choose>
            <xsl:when test="exists(for $i in $parent-facets[self::xs:maxInclusive] return $i) and exists(for $i in $child-facets[self::xs:maxExclusive] return $i)">
                <xsl:sequence select="for $i in $child-facets[self::xs:maxExclusive] return $i" />
            </xsl:when>
            <xsl:when test="exists(for $i in $parent-facets[self::xs:maxExclusive] return $i) and exists(for $i in $child-facets[self::xs:maxInclusive] return $i)">
                <xsl:sequence select="for $i in $child-facets[self::xs:maxInclusive] return $i" />
            </xsl:when>
            <xsl:otherwise>
                <xsl:sequence select="s:AND-facets(for $i in $parent-facets[self::xs:maxInclusive] return $i, for $i in $child-facets[self::xs:maxInclusive] return $i)" />
                <xsl:sequence select="s:AND-facets(for $i in $parent-facets[self::xs:maxExclusive] return $i, for $i in $child-facets[self::xs:maxExclusive] return $i)" />
            </xsl:otherwise>
        </xsl:choose>
        
        <xsl:sequence select="s:AND-facets(for $i in $parent-facets[self::xs:totalDigits] return $i, for $i in $child-facets[self::xstotalDigits] return $i)" />
        <xsl:sequence select="s:AND-facets(for $i in $parent-facets[self::xs:fractionDigits] return $i, for $i in $child-facets[self::xs:fractionDigits] return $i)" />
        
        <xsl:sequence select="s:AND-facets(for $i in $parent-facets[self::xs:enumeration] return $i, for $i in $child-facets[self::xs:enumeration] return $i)" />
        
        <xsl:sequence select="s:AND-facets(for $i in $parent-facets[self::xs:pattern] return $i, for $i in $child-facets[self::xs:pattern] return $i)" />
        
        
    </xsl:function>  
    
    <!-- 
        AND-facets :: Facet -> Facet -> Facet
    -->
    <xsl:function name="s:AND-facets">
        <xsl:param name="parent-facet" />
        <xsl:param name="child-facet" />
        
        <xsl:choose>
            
            <xsl:when test="exists($parent-facet) and exists($child-facet)">
                <xsl:sequence select="$child-facet" />
            </xsl:when>
            
            <xsl:when test="exists($parent-facet) and empty($child-facet)">
                <xsl:sequence select="$parent-facet" />
            </xsl:when>
            
            <xsl:when test="empty($parent-facet) and exists($child-facet)">
                <xsl:sequence select="$child-facet" />
            </xsl:when>
            
            <xsl:otherwise />
            
        </xsl:choose>
        
    </xsl:function>  
    
    <xsl:function name="s:sans-name-and-id">
        <xsl:param name="type"/>
        
        <xsl:for-each select="$type">
            <xsl:copy>
                <xsl:copy-of select="@* except (@name, @_id)"/>
                <xsl:sequence select="xs:* except (xs:annotation, xs:comment, xs:processing-instruction)"/>
            </xsl:copy>
        </xsl:for-each>
        
    </xsl:function>

</xsl:stylesheet>
