<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xs="http://www.w3.org/2001/XMLSchema"
                xmlns:h="http://www.schema-library.org/input-schema-and-its-dependency-hierarchy"
                exclude-result-prefixes="#all"
                version="2.0">
    
    <!-- 
        This program adds an id attribute
        onto each global element, attribute, simpleType, complexType,
        group, and attributeGroup. What's the value of doing this?
        Simple, only global stuff can be referenced, so this program
        prepares them to be part of an id-idref pair. See add-idref-substitutionGroupRef-and-targetNamespace for where
        I insert the idref attributes.
        
        Roger L. Costello
    -->
    
    <xsl:function name="h:add-id-to-top-level-items" as="document-node()">
        <xsl:param name="schemas" as="document-node()" />
        
        <xsl:variable name="updated-schemas">
            <xsl:apply-templates select="$schemas" mode="add-id-to-each-item"/>
        </xsl:variable>
        
        <xsl:sequence select="$updated-schemas" />
        
    </xsl:function>
    
    <xsl:template match="@* | node()" mode="add-id-to-each-item">
        
        <xsl:copy>
            <xsl:apply-templates select="@*"  mode="add-id-to-each-item" />
            <xsl:apply-templates  mode="add-id-to-each-item" />
        </xsl:copy>
        
    </xsl:template>
    
    <xsl:template match="*[parent::xs:schema][not(self::xs:annotation)][not(self::xs:import)][not(self::xs:include)][not(self::xs:redefine)][not(self::xs:key)][not(self::xs:keyref)][not(self::xs:unique)]" mode="add-id-to-each-item">
        
        <xsl:variable name="_id" select="generate-id(.)" />
        <xsl:copy>
            <xsl:apply-templates select="@*"  mode="add-id-to-each-item" />
            <xsl:attribute name="_id"><xsl:value-of select="$_id" /></xsl:attribute>
            <xsl:apply-templates  mode="add-id-to-each-item" />
        </xsl:copy>
        
    </xsl:template>
    
</xsl:stylesheet>