<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xs="http://www.w3.org/2001/XMLSchema"
                xmlns:h="http://www.schema-library.org/input-schema-and-its-dependency-hierarchy"
                exclude-result-prefixes="#all"
                version="2.0">
    
    <!-- 
        Suppose a schema has an
        include element that references schema A. Recall that each schema
        has an id attribute. This program will add an attribute on the
        include element, idref, with a value that matches the id attribute.
        Ditto for all import elements and redefine elements.
        
        Author: Roger L. Costello
    -->
    
    <xsl:function name="h:add-idref-on-includes-imports-redefines" as="document-node()">
        <xsl:param name="schemas" as="document-node()" />
        
        <xsl:variable name="updated-schemas">
            <xsl:apply-templates select="$schemas/schemas" mode="add-ref-to-include-import-redefine"/>
        </xsl:variable>
        
        <xsl:sequence select="$updated-schemas" />
        
    </xsl:function>
    
    <xsl:template match="@* | node()" mode="add-ref-to-include-import-redefine">
        
        <xsl:copy>
            <xsl:apply-templates select="@*"  mode="add-ref-to-include-import-redefine" />
            <xsl:apply-templates  mode="add-ref-to-include-import-redefine" />
        </xsl:copy>
        
    </xsl:template>
    
    
    <xsl:template match="xs:include[@schemaLocation]" mode="add-ref-to-include-import-redefine"> 
        <xsl:sequence select="h:add-ref(.)" />
    </xsl:template>
    
    
    <xsl:template match="xs:import[@schemaLocation]" mode="add-ref-to-include-import-redefine"> 
        <xsl:sequence select="h:add-ref(.)" />
    </xsl:template>
    
    
    <xsl:template match="xs:redefine[@schemaLocation]" mode="add-ref-to-include-import-redefine"> 
        <xsl:sequence select="h:add-ref(.)" />
    </xsl:template>
    
    <xsl:function name="h:add-ref">
        <xsl:param name="item" as="element()" />
        
        <xsl:for-each select="$item">
            <xsl:variable name="referenced-schema" select="document(@schemaLocation, .)/xs:schema" />
            <xsl:variable name="id-of-referenced-schema" select="generate-id($referenced-schema)" />
            <xsl:copy>
                <xsl:copy-of select="@*" />
                <xsl:attribute name="idref"><xsl:value-of select="$id-of-referenced-schema" /></xsl:attribute>
                <xsl:apply-templates mode="add-ref-to-include-import-redefine" />
            </xsl:copy>
        </xsl:for-each>
        
    </xsl:function>
    
</xsl:stylesheet>